<?php

namespace Drupal\xray_audit\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\xray_audit\Plugin\XrayAuditTaskPluginInterface;
use Drupal\xray_audit\Services\PluginRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Returns responses for XrayAudit Task routes.
 */
final class XrayAuditTaskController extends ControllerBase {

  /**
   * Plugin repository.
   *
   * @var \Drupal\xray_audit\Services\PluginRepositoryInterface
   */
  protected $pluginRepository;

  /**
   * Constructs of Controller.
   *
   * @param \Drupal\xray_audit\Services\PluginRepositoryInterface $plugin_repository
   *   Plugin manager for Groups.
   */
  public function __construct(PluginRepositoryInterface $plugin_repository) {
    $this->pluginRepository = $plugin_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('xray_audit.plugin_repository')
    );
  }

  /**
   * Return a render array with the results of the task.
   *
   * @param string $group_plugin
   *   Group plugin.
   * @param string $task_plugin_operation
   *   Task plugin operation.
   *
   * @return array
   *   Render array.
   */
  public function build(string $group_plugin, string $task_plugin_operation): array {
    $task_plugin_operation_explode = $this->getParameterTaskPluginOperationExplode($task_plugin_operation);

    /** @var string $task_plugin */
    $task_plugin = NULL;

    /** @var string $operation */
    $operation = NULL;

    // Extract and generate the variables $task_plugin and $operation.
    extract($task_plugin_operation_explode);

    $task_plugin = $this->getTaskPluginParamenter($task_plugin);
    $data = $task_plugin->getDataOperationResult($operation);
    return $task_plugin->buildDataRenderArray($data, $operation);

  }

  /**
   * Init a batch process.
   *
   * @param string $task_plugin
   *   Task plugin.
   * @param string $batch_process
   *   Batch proces.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect response if the batch is progressive. No return value otherwise.
   */
  public function buildBatchProcess(string $task_plugin, string $batch_process): RedirectResponse {
    $task_plugin = $this->getTaskPluginParamenter($task_plugin);

    $bacth_method = $task_plugin->getBatchClass($batch_process);
    if ($bacth_method === NULL || !method_exists($task_plugin, $bacth_method)) {
      throw new NotFoundHttpException();
    }

    return $task_plugin->$bacth_method();
  }

  /**
   * Return a render array with the results of the task.
   *
   * @param string $group_plugin
   *   Group plugin.
   * @param string $task_plugin_operation
   *   Task plugin operation.
   *
   * @return string
   *   Title.
   */
  public function getTitle(string $group_plugin, string $task_plugin_operation = NULL): string {
    $title = '';

    $task_plugin_operation_explode = $this->pluginRepository->explodeFromParameterTaskOperation($task_plugin_operation);
    if ($task_plugin_operation_explode === NULL) {
      $title = '';
    }

    /** @var string $task_plugin */
    $task_plugin = NULL;

    /** @var string $operation */
    $operation = NULL;

    // Extract and generate the variables $task_plugin and $operation.
    extract($task_plugin_operation_explode);

    $task_plugin = $this->pluginRepository->getInstancePluginTask($task_plugin);
    if ($task_plugin === NULL) {
      return $title;
    }

    $title .= $task_plugin->label();
    if ($operation) {
      $title .= ' - ' . ($task_plugin->getOperations()[$operation]['label'] ?? '');
    }
    return $title;
  }

  /**
   * Gets the $task_plugin_operation_explode parameter.
   *
   * @return array
   *   Parameters.
   */
  public function getParameterTaskPluginOperationExplode(string $task_plugin_operation): array {
    $task_plugin_operation_explode = $this->pluginRepository->explodeFromParameterTaskOperation($task_plugin_operation);
    if ($task_plugin_operation_explode === NULL) {
      throw new NotFoundHttpException();
    }
    return $task_plugin_operation_explode;
  }

  /**
   * Gets the $task_plugin parameter.
   *
   * @param string $task_plugin
   *   Task plugin.
   *
   * @return \Drupal\xray_audit\Plugin\XrayAuditTaskPluginInterface
   *   Plugin.
   */
  public function getTaskPluginParamenter($task_plugin): XrayAuditTaskPluginInterface {
    $task_plugin = $this->pluginRepository->getInstancePluginTask($task_plugin);
    if ($task_plugin === NULL) {
      throw new NotFoundHttpException();
    }
    return $task_plugin;
  }

}
